<?php

/**
 * @file
 * This file contains the class Services.
 */


namespace Drupal\webpay;

use GuzzleHttp\Exception\GuzzleException;
use Transbank\Webpay\Options;
use Transbank\Webpay\WebpayPlus\Exceptions\TransactionCommitException;
use Transbank\Webpay\WebpayPlus\Exceptions\TransactionCreateException;
use Transbank\Webpay\WebpayPlus\Transaction;

/**
 * The services Webpay Normal.
 */
class WebpayNormalService implements WebpayNormalServiceInterface {

  /**
   * Records errors messages.
   */
  protected $errors = [];

  /**
   * The config object of a commerce code.
   */
  protected $config;

  /**
   * @var \Transbank\Webpay\WebpayPlus\Transaction
   */
  protected $webpayTransactionService;

  /**
   * The commerce system used.
   */
  protected $commerceSystem;

  /**
   *
   * @param object $config
   * @param string $commerceSystem
   */
  public function __construct($config, $commerceSystem = '') {
    $this->config = $config;
    $options = new Options(
      $config->api_key,
      $config->commerce_code,
      $config->environment == WEBPAY_ENVIRONMENT_PROD ? Options::ENVIRONMENT_PRODUCTION : Options::ENVIRONMENT_INTEGRATION
    );
    $this->webpayTransactionService = new Transaction($options);
    $this->commerceSystem = $commerceSystem;
  }

  /**
   * {@inheritdoc}
   */
  public function initTransaction($buyOrder, $amount, $return_url, $instanceId, $session = NULL) {
    if (is_null($session)) {
      global $user;

      $session = $user->uid;
    }
    try {
      return $this->webpayTransactionService->create($buyOrder, $this->commerceSystem . ':' . $session, $amount, $return_url);
    } catch (GuzzleException $e) {
      return FALSE;
    } catch (TransactionCreateException $e) {
      watchdog('webpay', 'A init transaction error appears: %error', ['%error' => $e->getMessage()], WATCHDOG_ERROR);
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function isValidRequest() {
    return !$this->isInvalidRequest();
  }

  /**
   * {@inheritdoc}
   */
  public function isInvalidRequest() {
    $params = drupal_get_query_parameters();
    $token_ws = $params['token_ws'] ?? '';
    $tbk_token = $params['TBK_TOKEN'] ?? '';
    $buy_order = $params['TBK_ORDEN_COMPRA'] ?? '';
    $session_id = $params['TBK_ID_SESION'] ?? '';

    if ($tbk_token && $buy_order && $session_id && !$token_ws) {
      return self::USER_ABORTED_PAYMENT;
    }
    if ($token_ws && $buy_order && $session_id && $tbk_token) {
      return self::SOME_ERROR_ON_WEBPAY;
    }
    if ($buy_order && $session_id && !$token_ws && !$tbk_token) {
      return self::USER_WAS_IDLE;
    }
    if (!($token_ws && !$buy_order && !$session_id && !$tbk_token)) {
      return self::IS_NOT_A_NORMAL_RESPONSE;
    }

    return FALSE;
  }

  /**
   * @return false|\Drupal\webpay\Entity\WebpayTransaction
   */
  public function commit() {
    $params = drupal_get_query_parameters();
    $token_ws = $params['token_ws'] ?? '';
    try {
      $response = $this->webpayTransactionService->commit($token_ws);
    } catch (GuzzleException $e) {
      return FALSE;
    } catch (TransactionCommitException $e) {
      watchdog('webpay', 'A commit error appears: %error', ['%error' => $e->getMessage()], WATCHDOG_ERROR);
      return FALSE;
    }

    if ($transaction = webpay_get_transaction_by_token($token_ws)) {
      return $transaction;
    }

    $transaction = entity_create('webpay_transaction', array(
      'config_id' => $this->config->id,
      'commerce_code' => $this->config->commerce_code,
      'token' => $token_ws,
      'order_number' => $response->getBuyOrder(),
      'session_id' => $response->getSessionId(),
      'transaction_date' => strtotime($response->getTransactionDate()),
      'vci' => $response->getVci(),
      'card_number' => $response->getCardNumber(),
      'authorization_code' => $response->getAuthorizationCode(),
      'payment_type_code' => $response->getPaymentTypeCode(),
      'response_code' => $response->getResponseCode(),
      'amount' => $response->getAmount(),
      'shares_number' => $response->getInstallmentsNumber(),
    ));

    try {
      $transaction->save();
    } catch (\Exception $err) {
      $watchdog_message = 'Failed to save the transaction in the database: @err';
      $watchdog_variable = array('@err' => $err->getMessage());
      watchdog('WEBPAY', $watchdog_message, $watchdog_variable, WATCHDOG_ERROR);

      return FALSE;
    }

    return $transaction;
  }
}
